# Original file by: Christopher Harvey

CMAKE_MINIMUM_REQUIRED(VERSION 2.6 FATAL_ERROR) #uses FindPackageHandleStdArgs
INCLUDE(${CMAKE_ROOT}/Modules/FindPackageHandleStandardArgs.cmake)


# Our initial guess will be within the SDK.
set(FREEIMAGE_INSTALL_DIR "${CMAKE_SOURCE_DIR}/../FreeImage/" CACHE PATH "Path to FreeImage installed location.")

# The distribution contains both 32 and 64 bit libraries.  Adjust the library
# search path based on the bit-ness of the build.  (i.e. 64: bin64, lib64; 32:
# bin, lib).  Note that on Mac, the OptiX library is a universal binary, so we
# only need to look in lib and not lib64 for 64 bit builds.
####
# Pasted from FindOptiX.cmake, but I can not test apple. linux will be tested later.
if(CMAKE_SIZEOF_VOID_P EQUAL 8)
  set(bit_dest "64")
else()
  set(bit_dest "")
endif()

macro(FreeImage_find_api_library name)
  find_library(${name}_LIBRARY
    NAMES ${name}
    PATHS "${FREEIMAGE_INSTALL_DIR}/lib${bit_dest}"
    NO_DEFAULT_PATH
    )
  find_library(${name}_LIBRARY
    NAMES ${name}
    )
	#MESSAGE("${name}_LIBRARY: ${${name}_LIBRARY}")
  if(WIN32)
    find_file(${name}_DLL
      NAMES ${name}.dll
      PATHS "${FREEIMAGE_INSTALL_DIR}/bin${bit_dest}"
      NO_DEFAULT_PATH
      )
    find_file(${name}_DLL
      NAMES ${name}.dll
      )
	#MESSAGE("${name}_DLL: ${${name}_DLL}")
  endif()
endmacro()

FreeImage_find_api_library(FreeImage)

FIND_PATH(FREEIMAGE_INCLUDE FreeImage.h
  PATHS
  ${FREEIMAGE_INSTALL_DIR}/include
  DOC "The path to the directory that contains FreeImage.h"
)
#MESSAGE("FREEIMAGE_INCLUDE: ${FREEIMAGE_INCLUDE}")

if(WIN32)
  FIND_PACKAGE_HANDLE_STANDARD_ARGS(FreeImage DEFAULT_MSG 
                                  FreeImage_DLL FreeImage_LIBRARY FREEIMAGE_INCLUDE)
else()
  FIND_PACKAGE_HANDLE_STANDARD_ARGS(FreeImage DEFAULT_MSG 
                                  FreeImage_LIBRARY FREEIMAGE_INCLUDE)
endif()

# Macro for setting up dummy targets
####
# Taken from FindOptiX.cmake
function(FreeImage_add_imported_library name lib_location dll_lib dependent_libs)
  set(CMAKE_IMPORT_FILE_VERSION 1)

  # Create imported target
  add_library(${name} SHARED IMPORTED)

  # Import target "optix" for configuration "Debug"
  if(WIN32)
    set_target_properties(${name} PROPERTIES
      IMPORTED_IMPLIB "${lib_location}"
      #IMPORTED_LINK_INTERFACE_LIBRARIES "glu32;opengl32"
      IMPORTED_LOCATION "${dll_lib}"
      IMPORTED_LINK_INTERFACE_LIBRARIES "${dependent_libs}"
      )
  elseif(UNIX)
    set_target_properties(${name} PROPERTIES
      #IMPORTED_LINK_INTERFACE_LIBRARIES "glu32;opengl32"
      IMPORTED_LOCATION "${lib_location}"
      # We don't have versioned filenames for now, and it may not even matter.
      #IMPORTED_SONAME "${optix_soname}"
      IMPORTED_LINK_INTERFACE_LIBRARIES "${dependent_libs}"
      )
  else()
    # Unknown system, but at least try and provide the minimum required
    # information.
    set_target_properties(${name} PROPERTIES
      IMPORTED_LOCATION "${lib_location}"
      IMPORTED_LINK_INTERFACE_LIBRARIES "${dependent_libs}"
      )
  endif()

  # Commands beyond this point should not need to know the version.
  set(CMAKE_IMPORT_FILE_VERSION)
endfunction()

FreeImage_add_imported_library(FreeImage   "${FreeImage_LIBRARY}"   "${FreeImage_DLL}"   "")
