#include <optix_world.h>
#include "random.h"

using namespace optix;

struct PerRayData_pathtrace
{
  float3 result;
};

// Scene wide
rtDeclareVariable(float,         scene_epsilon, , );
rtDeclareVariable(rtObject,      top_object, , );
rtDeclareVariable(uint2,         launch_index, rtLaunchIndex, );
rtDeclareVariable(uint,          tileX, , );
rtDeclareVariable(uint,          tileY, , );
rtDeclareVariable(uint,          tileSize, , );

// camera
rtDeclareVariable(float3,        eye, , );
rtDeclareVariable(float3,        U, , );
rtDeclareVariable(float3,        V, , );
rtDeclareVariable(float3,        W, , );
rtDeclareVariable(unsigned int,  frameNumber, , );
rtBuffer<float4, 2>              output_buffer;

rtDeclareVariable(unsigned int,  pathtrace_ray_type, , );

// For miss program
rtDeclareVariable(float3,       bg_color, , );
rtDeclareVariable(PerRayData_pathtrace, current_prd, rtPayload, );

// for exception program
rtDeclareVariable(float3,        bad_color, , );




__device__ float filterTent___(float var) {
    if(var < 0.5f) {
        return var + 1.f;
    }
    else {
        return 1.f - var;
    }
}

__device__ float filterValue___(const float2& rayOrigin, const uint2& pixel) {
    float2 relativePosition = rayOrigin - make_float2(pixel.x, pixel.y) + make_float2(0.5f);

    float hx = filterTent___(relativePosition.x);
    float hy = filterTent___(relativePosition.y);
    return hx * hy;
}


RT_PROGRAM void pathtrace_camera()
{

    uint2 screenPos = make_uint2(launch_index.x, launch_index.y);
    float2 screenRayPos = make_float2(screenPos) + make_float2(0.5f);

    size_t2 screen = output_buffer.size();

    float2 inv_screen = 1.0f/make_float2(screen);
    float2 pixel = (make_float2(screenPos)+0.5f) * 2.f * inv_screen - 1.f;


    unsigned int seed = tea<16>(screen.x*screenPos.y+screenPos.x, frameNumber);
    float2 jitter = make_float2(rnd(seed), rnd(seed));
    jitter -= make_float2(0.5f, 0.5f);
    screenRayPos += jitter;


    float2 d = pixel + jitter * 2.f * inv_screen;   // device space [-1, 1; -1, 1], same scaling as with screen space
    float3 ray_origin = eye;
    float3 ray_direction = normalize(d.x*U + d.y*V + W);

    PerRayData_pathtrace prd;
    prd.result = make_float3(0.f);

    Ray ray = make_Ray(ray_origin, ray_direction, pathtrace_ray_type, scene_epsilon, RT_DEFAULT_MAX);
    rtTrace(top_object, ray, prd);

    float filter = 1.f;
    filter = filterValue___(screenRayPos, screenPos); // program compiles when removing (commenting out) this line

    float3 eyeImage = prd.result * filter;
    output_buffer[screenPos] = make_float4(eyeImage, filter);

}

//-----------------------------------------------------------------------------
//
//  Exception program
//
//-----------------------------------------------------------------------------

RT_PROGRAM void exception()
{
  output_buffer[launch_index] = make_float4(bad_color, 0.0f);
}


//-----------------------------------------------------------------------------
//
//  Miss program
//
//-----------------------------------------------------------------------------

RT_PROGRAM void miss()
{
  current_prd.result = bg_color;
}

RT_PROGRAM void pathTracer()
{

}

