
/*
 * Copyright (c) 2008 - 2009 NVIDIA Corporation.  All rights reserved.
 *
 * NVIDIA Corporation and its licensors retain all intellectual property and proprietary
 * rights in and to this software, related documentation and any modifications thereto.
 * Any use, reproduction, disclosure or distribution of this software and related
 * documentation without an express license agreement from NVIDIA Corporation is strictly
 * prohibited.
 *
 * TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED *AS IS*
 * AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR IMPLIED,
 * INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS BE LIABLE FOR ANY
 * SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES WHATSOEVER (INCLUDING, WITHOUT
 * LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, LOSS OF
 * BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS) ARISING OUT OF THE USE OF OR
 * INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGES
 */

#include <ImageLoader.h>
#include <PPMLoader.h>
#include <HDRLoader.h>
#include <fstream>
#include <optixu/optixu_math_namespace.h>

#include <iostream>

//-----------------------------------------------------------------------------
//  
//  Utility functions 
//
//-----------------------------------------------------------------------------

optix::TextureSampler defaultColour(optix::Context context, const optix::float3& default_color) {
    optix::TextureSampler sampler = context->createTextureSampler();
    sampler->setWrapMode( 0, RT_WRAP_REPEAT );
    sampler->setWrapMode( 1, RT_WRAP_REPEAT );
    sampler->setWrapMode( 2, RT_WRAP_REPEAT );
    sampler->setIndexingMode( RT_TEXTURE_INDEX_NORMALIZED_COORDINATES );
    sampler->setReadMode( RT_TEXTURE_READ_NORMALIZED_FLOAT );
    sampler->setMaxAnisotropy( 1.0f );
    sampler->setMipLevelCount( 1u );
    sampler->setArraySize( 1u );

    // Create buffer with single texel set to default_color
    optix::Buffer buffer = context->createBuffer( RT_BUFFER_INPUT, RT_FORMAT_UNSIGNED_BYTE4, 1u, 1u );
    unsigned char* buffer_data = static_cast<unsigned char*>( buffer->map() );
    buffer_data[0] = (unsigned char) optix::clamp((int)(default_color.x * 255.0f), 0, 255);
    buffer_data[1] = (unsigned char) optix::clamp((int)(default_color.y * 255.0f), 0, 255);
    buffer_data[2] = (unsigned char) optix::clamp((int)(default_color.z * 255.0f), 0, 255);
    buffer_data[3] = 255;
    buffer->unmap();

    sampler->setBuffer( 0u, 0u, buffer );
    // Although it would be possible to use nearest filtering here, we chose linear
    // to be consistent with the textures that have been loaded from a file. This
    // allows OptiX to perform some optimizations.
    sampler->setFilteringModes( RT_FILTER_LINEAR, RT_FILTER_LINEAR, RT_FILTER_NONE );

    return sampler;
}

SUTILAPI optix::TextureSampler loadTexture( optix::Context context,
                                            const std::string& filename,
                                            const optix::float3& default_color,
                                            RTwrapmode wrapmode)
{

    bool IsHDR = false;
    size_t len = filename.length();
    if(len >= 3) {
        IsHDR = (filename[len-3] == 'H' || filename[len-3] == 'h') &&
                (filename[len-2] == 'D' || filename[len-2] == 'd') &&
                (filename[len-1] == 'R' || filename[len-1] == 'r');
    }
    if(IsHDR)
        return loadHDRTexture(context, filename, default_color);
    else {
        return loadPPMTexture(context, filename, default_color);
    }
}

